/**
	******************************************************************************
	* @file 		Buttons_FT32F072_Starter_kit.c
	* @author 	FMD AE
	* @brief 		Buttons interface for Starter_kit_072
	* @version 	V1.0.0
	* @data 		2021-09-15
	******************************************************************************
	* @attention
	* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
	* This software is provided by the copyright holders and contributors,and the
	*	software is believed to be accurate and reliable. However, Fremont Micro
	*	Devices (SZ) Corporation assumes no responsibility for the consequences of
	*	use of such software or for any infringement of patents of other rights
	*	of third parties, which may result from its use. No license is granted by
	*	implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
	*	Corporation.
	******************************************************************************
	*/

/* Includes ------------------------------------------------------------------*/
#include "Board_Buttons.h"                  // ::Board Support:LED
#include "ft32f0xx.h" 

typedef struct {
	uint32_t 			RCC_AHBPeriph;
  GPIO_TypeDef *port;
  uint16_t      pin;
  BitAction 		active_state;
  uint8_t       reserved;
} PIN_CONFIG_t;

/* Private defines -----------------------------------------------------------*/
#define	BUTTON1_CLK		RCC_AHBENR_GPIOCEN
#define	BUTTON2_CLK		RCC_AHBENR_GPIOCEN

#define	BUTTON1_GPIOX	GPIOC
#define	BUTTON2_GPIOX	GPIOC

#define	BUTTON1_GPIO_Pin	GPIO_Pin_13
#define	BUTTON2_GPIO_Pin	GPIO_Pin_11
/* ------------------------------- Configuration ---------------------------- */
#define BUTTON_NUM                         2U

static const PIN_CONFIG_t PIN_CONFIG[BUTTON_NUM] = {
    { BUTTON1_CLK, BUTTON1_GPIOX, BUTTON1_GPIO_Pin, Bit_SET, 0U }
  , { BUTTON2_CLK, BUTTON2_GPIOX, BUTTON2_GPIO_Pin, Bit_RESET, 0U }
};
/* -------------------------------------------------------------------------- */

/**
  \fn          int32_t Buttons_Initialize (void)
  \brief       Initialize I/O interface for buttons
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Buttons_Initialize (void) {

	GPIO_InitTypeDef	GPIO_InitStructure;

	/* BUTTONS GPIO Periph clock enable */
	RCC_AHBPeriphClockCmd(PIN_CONFIG[0].RCC_AHBPeriph,ENABLE);
	RCC_AHBPeriphClockCmd(PIN_CONFIG[1].RCC_AHBPeriph,ENABLE);
	
	/* Configure BUTTON1_GPIO_Pin in input pull down mode */
	GPIO_InitStructure.GPIO_Pin = PIN_CONFIG[0].pin;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
	
	GPIO_Init(PIN_CONFIG[0].port,&GPIO_InitStructure);
	
	/* Configure BUTTON2_GPIO_Pin in input pull up mode */
	GPIO_InitStructure.GPIO_Pin = PIN_CONFIG[1].pin;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	
	GPIO_Init(PIN_CONFIG[1].port,&GPIO_InitStructure);
	
  return 0;
}

/**
  \fn          int32_t Buttons_Uninitialize (void)
  \brief       De-initialize I/O interface for buttons
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Buttons_Uninitialize (void) {

	uint8_t i;
	
	for (i = 0U; i < BUTTON_NUM; i++){
		GPIO_DeInit(PIN_CONFIG[i].port);
	}	

  return 0;
}


/**
  \fn          uint32_t Buttons_GetState (void)
  \brief       Get buttons state
  \returns     Buttons state
*/
uint32_t Buttons_GetState (void) {
  uint32_t val;
  uint8_t  i;

  val = 0U;
  for (i = 0U; i < BUTTON_NUM; i++) {
    if (GPIO_ReadInputDataBit(PIN_CONFIG[i].port, PIN_CONFIG[i].pin) == PIN_CONFIG[i].active_state) {
      val |= 1UL << i;
    }
  }

  return val;
}

/**
  \fn          uint32_t Buttons_GetCount (void)
  \brief       Get number of available buttons
  \return      Number of available buttons
*/
uint32_t Buttons_GetCount (void) {
  return BUTTON_NUM;
}

