/**
	******************************************************************************
	* @file 		LED_FT32F072_Starter_kit.c
	* @author 	FMD AE
	* @brief 		LED interface for Starter_kit_072
	* @version 	V1.0.0
	* @data 		2021-09-15
	******************************************************************************
	* @attention
	* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
	* This software is provided by the copyright holders and contributors,and the
	*	software is believed to be accurate and reliable. However, Fremont Micro
	*	Devices (SZ) Corporation assumes no responsibility for the consequences of
	*	use of such software or for any infringement of patents of other rights
	*	of third parties, which may result from its use. No license is granted by
	*	implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
	*	Corporation.
	******************************************************************************
	*/

/* Includes ------------------------------------------------------------------*/
#include "Board_LED.h"                  // ::Board Support:LED
#include "ft32f0xx.h" 

typedef struct {
	uint32_t 			RCC_AHBPeriph;
  GPIO_TypeDef *port;
  uint16_t      pin;
  BitAction 		active_state;
  uint8_t       reserved;
} PIN_CONFIG_t;

/* Private defines -----------------------------------------------------------*/
#define	LED1_CLK		RCC_AHBENR_GPIOBEN
#define	LED2_CLK		RCC_AHBENR_GPIOBEN
#define	LED3_CLK		RCC_AHBENR_GPIOBEN
#define	LED4_CLK		RCC_AHBENR_GPIOBEN

#define	LED1_GPIOX	GPIOB
#define	LED2_GPIOX	GPIOB
#define	LED3_GPIOX	GPIOB
#define	LED4_GPIOX	GPIOB

#define	LED1_GPIO_Pin	GPIO_Pin_0
#define	LED2_GPIO_Pin	GPIO_Pin_1
#define	LED3_GPIO_Pin	GPIO_Pin_2
#define	LED4_GPIO_Pin	GPIO_Pin_3

/* ------------------------------- Configuration ---------------------------- */
#define LED_NUM                         4U

static const PIN_CONFIG_t PIN_CONFIG[LED_NUM] = {
    { LED1_CLK, LED1_GPIOX, LED1_GPIO_Pin, Bit_RESET, 0U }
  , { LED2_CLK, LED2_GPIOX, LED2_GPIO_Pin, Bit_RESET, 0U }
  , { LED3_CLK, LED3_GPIOX, LED3_GPIO_Pin, Bit_SET, 0U }
  , { LED4_CLK, LED4_GPIOX, LED4_GPIO_Pin, Bit_SET, 0U }
};
/* -------------------------------------------------------------------------- */

/**
  \fn          int32_t LED_Initialize (void)
  \brief       Initialize I/O interface for LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Initialize (void) {

	uint8_t i;
	
	GPIO_InitTypeDef	GPIO_InitStructure;

	/* Configure LED1_GPIO_Pin,LED2_GPIO_Pin,LED3_GPIO_Pin and LED4_GPIO_Pin in output pushpull mode */
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	
	for (i = 0U; i < LED_NUM; i++){
		
		GPIO_InitStructure.GPIO_Pin = PIN_CONFIG[i].pin;
		
		/* LED GPIO Periph clock enable */
		RCC_AHBPeriphClockCmd(PIN_CONFIG[i].RCC_AHBPeriph,ENABLE);
		GPIO_Init(PIN_CONFIG[i].port,&GPIO_InitStructure);
	}
	
	
	/* switch LEDs off */
	LED_SetOut(0);
	
  return 0;
}

/**
  \fn          int32_t LED_Uninitialize (void)
  \brief       De-initialize I/O interface for LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Uninitialize (void) {

	uint8_t i;
	
	for (i = 0U; i < LED_NUM; i++){
		GPIO_DeInit(PIN_CONFIG[i].port);
	}	

  return 0;
}

/**
  \fn          int32_t LED_On (uint32_t num)
  \brief       Turn on requested LED
  \param[in]   num  LED number
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_On (uint32_t num) {

  if (num >= LED_NUM) {
    return -1;
  }

  if (PIN_CONFIG[num].active_state == Bit_RESET) {
    GPIO_WriteBit(PIN_CONFIG[num].port, PIN_CONFIG[num].pin, Bit_RESET);
  } else {
    GPIO_WriteBit(PIN_CONFIG[num].port, PIN_CONFIG[num].pin, Bit_SET);
  }

  return 0;
}

/**
  \fn          int32_t LED_Off (uint32_t num)
  \brief       Turn off requested LED
  \param[in]   num  LED number
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Off (uint32_t num) {

  if (num >= LED_NUM) {
    return -1;
  }

  if (PIN_CONFIG[num].active_state == Bit_RESET) {
    GPIO_WriteBit(PIN_CONFIG[num].port, PIN_CONFIG[num].pin, Bit_SET);
  } else {
    GPIO_WriteBit(PIN_CONFIG[num].port, PIN_CONFIG[num].pin, Bit_RESET);
  }

  return 0;
}

/**
  \fn          int32_t LED_SetOut (uint32_t val)
  \brief       Write value to LEDs
  \param[in]   val  value to be displayed on LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_SetOut(uint32_t value) {
  uint8_t i;

  for (i = 0U; i < LED_NUM; i++) {
    if (value & (1UL << i)) {
      LED_On(i);
    } else {
      LED_Off(i);
    }
  }

  return 0;
}

/**
  \fn          uint32_t LED_GetCount (void)
  \brief       Get number of LEDs
  \return      Number of available LEDs
*/
uint32_t LED_GetCount (void) {
  return LED_NUM;
}

