/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : board.c
* @author    : FMD AE
* @brief     : board program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "board.h"

/* Private Constant --------------------------------------------------------------*/
/* ADC reference voltage :unit 1mv */
const uint16_t ADC_REFERENCE_VOLTAGE = 2500;

/* Usart baud rate */
const uint32_t USART_BAUD_RATE = 115200;

/* TICK_RATE_HZ */
const uint32_t TICK_RATE_HZ = 100;

/* adc measure frequency:unit 10ms  */
const uint8_t ADC_MEASURE_FREQUENCY = 100;

/* Total number of ADC samples */


/* Corresponding ADC value at 12-bit resolution */
const uint16_t ADC_12BIT_VALUE = 0xFFF;

/* specifies the IWDG Reload value */
const uint16_t IWDG_RELOAD_VALUE = 500;

/* The value of the PWM register that is adjusted with each keystroke */
const uint16_t PWM_DUTY_STEP = 100;

/* Key anti-bounce detection time */
const uint8_t KEY_DEBOUNCE = 2;

#define ADC_SAMPLE_NUMBER 10
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* The value and voltage converted by an ADC */
__IO uint16_t  g_iAdc1ConvertedValue = 0, g_iAdc1ConvertedVoltage = 0;

/* Channel1 duty cycle */
uint16_t g_iChannel1Pulse;

/* Period */
uint16_t g_iTimerPeriod;

/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
void LED_Toggle(GPIO_TypeDef *GPIOx, uint16_t GPIO_Pin);
KEY_DATA KEY_Scan(void);
void DMA_RxConfig(void);

/* Public function ------ --------------------------------------------------------*/

/******************************************************************************
  * @brief  LED_Init program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void LED_Init(void)
{
    GPIO_InitTypeDef    GPIO_InitStructure;

    /* GPIOB Clocks enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    /* Configure LED GPIO as push-pull output */
    GPIO_InitStructure.GPIO_Pin = LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(LED_PORT, &GPIO_InitStructure);

    /* Turn off LEDS*/
    LED_OFF;

}

/******************************************************************************
  * @brief  KEY_Init program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void KEY_Init(void)
{
    GPIO_InitTypeDef    GPIO_InitStructure;

    /* GPIOC Clocks enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

    /* Configure KEY1   GPIO as  pull down input mode */
    GPIO_InitStructure.GPIO_Pin = KEY1_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
    GPIO_Init(KEY_PORT, &GPIO_InitStructure);

    /* Configure KEY2   in input pull up mode */
    GPIO_InitStructure.GPIO_Pin = KEY2_PIN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(KEY_PORT, &GPIO_InitStructure);

}

/******************************************************************************
  * @brief  SysTick_Configuration program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void SysTick_Configuration(void)
{
    /* Setup SysTick Timer for 10 msec interrupts. */
    if (SysTick_Config(SystemCoreClock / TICK_RATE_HZ))
    {
        while (1);
    }

    /* set the SysTick Clock source to be HCLK */
    SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);

    /* Configuration of the SysTick Interrupt priority */
    NVIC_SetPriority(SysTick_IRQn, 0x04);

}

/******************************************************************************
  * @brief  USART_Config program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USART_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

    /* GPIOA Clocks enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* USART1 Clocks enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

    /* Set PA9 to TX and PA10 to RX */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_1);

    /* Initializes the USART1 peripheral according to the specified parameters in the USART_InitStructure */
    USART_InitStructure.USART_BaudRate = USART_BAUD_RATE;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_Init(USART1, &USART_InitStructure);

    /* USART1 enable */
    USART_Cmd(USART1, ENABLE);

    /* Enable the USART1 global Interrupt */
    NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    USART_ClearFlag(USART1, USART_FLAG_IDLE);

    /* USART1 IDLE interrupt enable */
    USART_ITConfig(USART1, USART_IT_IDLE, ENABLE);

    /* Configure RX to use DMA*/
    DMA_RxConfig();

    /* Disables DMA requests when receiving errors is enabled */
    USART_DMAReceptionErrorConfig(USART1, USART_DMAOnError_Enable);

    /* Enable DMA requests */
    USART_DMACmd(USART1, USART_DMAReq_Rx, ENABLE);

    /* Enable DMA channels */
    DMA_Cmd(DMA1_Channel3, ENABLE);

}

/******************************************************************************
  * @brief  ADC_Basic_Config    program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void ADC_Config(void)
{

    GPIO_InitTypeDef    GPIO_InitStructure;
    ADC_InitTypeDef     ADC_InitStructure;

    /* GPIOC Clocks enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

    /* ADC Clocks enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

    /* Configure PC0 as the analog input */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 ;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
    GPIO_Init(GPIOC, &GPIO_InitStructure);

    /* Configure ADC Resolution, ConvMode, DataAlign, ScanDirection */
    ADC_DeInit(ADC1);
    ADC_StructInit(&ADC_InitStructure);
    ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
    ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
    ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_ScanDirection = ADC_ScanDirection_Upward;
    ADC_Init(ADC1, &ADC_InitStructure);

    /* Set reference voltage to 2.5V (If you want to configure the ADC reference voltage to VDD,\
    do not call the following function )*/
    ADC_VrefselConfig(ADC_Vrefsel_2_5V);

    /* Configures for the selected ADC and its sampling time. */
    ADC_ChannelConfig(ADC1, ADC_Channel_10, ADC_SampleTime_239_5Cycles);

    /* ADC Calibration */
    ADC_GetCalibrationFactor(ADC1);

    /* Enable the ADC peripheral */
    ADC_Cmd(ADC1, ENABLE);

    /* Wait the ADRDY flag */
    while (!ADC_GetFlagStatus(ADC1, ADC_FLAG_ADRDY));

    /* ADC1 regular Software Start Conv */
    ADC_StartOfConversion(ADC1);

}

/******************************************************************************
  * @brief  ADC_Measure program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void ADC_Measure(void)
{
    uint32_t adc_max;
    uint32_t adc_min;
    uint32_t adc_sum;
    uint32_t adc_buff[ADC_SAMPLE_NUMBER];

    /* The ADC is measured about 1s at a time */
    if (g_wTimeCount % ADC_MEASURE_FREQUENCY == 0)
    {
        for (uint8_t i = 0; i < ADC_SAMPLE_NUMBER; i++)
        {
            /* Test EOC flag */
            while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);

            /* Get ADC1 converted data */
            g_iAdc1ConvertedValue = ADC_GetConversionValue(ADC1);

            /* Compute the voltage (Reference voltage 2.5V, 2500mv) */
            adc_buff[i] = (g_iAdc1ConvertedValue * ADC_REFERENCE_VOLTAGE) / ADC_12BIT_VALUE;

        }

        /* Get the average value of the ADC sample and print it */
        adc_max = adc_buff[0];
        adc_min = adc_buff[0];
        adc_sum = 0;

        for (uint8_t i = 0; i < ADC_SAMPLE_NUMBER; i++)
        {
            adc_sum += adc_buff[i];
            if (adc_buff[i] > adc_max)
            {
                adc_max = adc_buff[i];
            }
            else if (adc_buff[i] < adc_min)
            {
                adc_min = adc_buff[i];
            }
        }
        g_iAdc1ConvertedVoltage = (adc_sum - adc_max - adc_min) / (ADC_SAMPLE_NUMBER - 2);
        printf("ADC_DATA = 0x%02x\r\n", g_iAdc1ConvertedVoltage);

    }

}
/******************************************************************************
  * @brief  TIM_Config  program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void TIM_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
    TIM_OCInitTypeDef  TIM_OCInitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

    /* GPIOA and GPIOB Clocks enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB, ENABLE);

    /* GPIOA Configuration: Channel 1 as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_2);

    /* GPIOB Configuration: Channel 1N as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_2);

    /* Compute the value to be set in ARR regiter to generate signal frequency at 20 Khz */
    g_iTimerPeriod = (SystemCoreClock / 20000) - 1;
    /* Compute CCR1 value to generate a duty cycle at 50% for channel 1 and 1N */
    g_iChannel1Pulse = (uint16_t)(((uint32_t) 5 * (g_iTimerPeriod - 1)) / 10);

    /* TIM1 clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);

    /* Time Base configuration */
    TIM_TimeBaseStructure.TIM_Prescaler = 0;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseStructure.TIM_Period = g_iTimerPeriod;
    TIM_TimeBaseStructure.TIM_ClockDivision = 0;
    TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
    TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

    /* Channel 1 Configuration in PWM mode */
    TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
    TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
    TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
    TIM_OCInitStructure.TIM_Pulse = g_iChannel1Pulse;
    TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
    TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_Low;
    TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Set;
    TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Reset;
    TIM_OC1Init(TIM1, &TIM_OCInitStructure);

    /* TIM1 counter enable */
    TIM_Cmd(TIM1, ENABLE);
    /* Enable the CC2 Interrupt Request */
    TIM_ITConfig(TIM1, TIM_IT_Update, ENABLE);

    /* Enable the TIM1 global Interrupt */
    NVIC_InitStructure.NVIC_IRQChannel = TIM1_BRK_UP_TRG_COM_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    /* TIM1 Main Output Enable */
    TIM_CtrlPWMOutputs(TIM1, ENABLE);

}
/******************************************************************************
  * @brief  IWDG_Init   program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void IWDG_Init(void)
{
    /* LSI Clocks enable */
    RCC_LSICmd(ENABLE);

    /* Wait LSI ready */
    while (RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET);

    /* Wait some time, and then configure the IWDG */
    for (uint32_t i = 1000; i > 0; i--)
    {
        __ASM("NOP");
    }

    /* IWDG enable*/
    IWDG_Enable();

    /* Open IWDG register access permissions */
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);

    /* Set up the IWDG predivider */
    IWDG_SetPrescaler(IWDG_Prescaler_16);

    /* Wait until the predivider value is updated */
    while (IWDG_GetFlagStatus(IWDG_FLAG_PVU) == SET);

    /* Set the overloaded register for IWDG */
    IWDG_SetReload(IWDG_RELOAD_VALUE);

    /* Wait for the overload value to finish updating */
    while (IWDG_GetFlagStatus(IWDG_FLAG_RVU) == SET);

    /* Loads the overloaded register value into the counter */
    IWDG_ReloadCounter();

}

/******************************************************************************
  * @brief  LED_Flash   program
    * @param  FlashRate:0-0XFF
  * @note
  * @retval None
  *****************************************************************************
*/
void LED_Flash(uint8_t FlashRate)
{
    if (g_wTimeCount % (FlashRate / 10) == 0)
    {
        LED_Toggle(LED_PORT, LED3_PIN);
    }
}
/******************************************************************************
  * @brief  KEY_Pro program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void KEY_Pro(void)
{
    uint8_t Key_Value;

    /* Get the key value */
    Key_Value = KEY_Scan();

    if (Key_Value)
    {
        switch (Key_Value)
        {
        case KEY1_Data:
        {
            /* Key 1 controls LED1 toggle and increases duty cycle */
            LED_Toggle(LED_PORT, LED1_PIN);
            if (g_iChannel1Pulse < g_iTimerPeriod - PWM_DUTY_STEP)
            {
                g_iChannel1Pulse += PWM_DUTY_STEP;
            }
            TIM_SetCompare1(TIM1, g_iChannel1Pulse);
            printf("KEY1 PRESS, PWMPeriod = %02x, PWMPulse = %02x\r\n", g_iTimerPeriod, g_iChannel1Pulse);
        }
        break;
        case KEY2_Data:
        {
            /* Key 2 controls LED2 toggle and reduces duty cycle */
            LED_Toggle(LED_PORT, LED2_PIN);
            if (g_iChannel1Pulse > PWM_DUTY_STEP)
            {
                g_iChannel1Pulse -= PWM_DUTY_STEP;
            }
            TIM_SetCompare1(TIM1, g_iChannel1Pulse);
            printf("KEY2 PRESS, PWMPeriod = %02x, PWMPulse = %02x\r\n", g_iTimerPeriod, g_iChannel1Pulse);
        }
        break;
        }
    }
}

/* Private function ------ -------------------------------------------------------*/

/******************************************************************************
  * @brief  LED_Toggle program.
  * @param  GPIOx: where x can be (A, B, C, D, E or F) to select the GPIO peripheral.
    * @param  GPIO_Pin: specifies the port bit to be written.
  * @note
  * @retval None
  *****************************************************************************
*/
void LED_Toggle(GPIO_TypeDef *GPIOx, uint16_t GPIO_Pin)
{
    GPIO_WriteBit(GPIOx, GPIO_Pin, (BitAction)((1 - GPIO_ReadOutputDataBit(GPIOx, GPIO_Pin))));
}

/******************************************************************************
  * @brief  KEY_Scan    program
  * @param  None
  * @note
  * @retval return key value
  *****************************************************************************
*/
KEY_DATA KEY_Scan(void)
{
    static uint8_t s_key_up = 1;
    static uint8_t s_key_time = 0;

    if (s_key_up && (KEY1_INPUT_READ == 1 || KEY2_INPUT_READ == 0))
    {
        /* key down detected , return key value*/
        s_key_time++;
        if (s_key_time >= KEY_DEBOUNCE)
        {
            s_key_up = 0;
            if (KEY1_INPUT_READ == 1)
            {
                return KEY1_Data;
            }
            if (KEY2_INPUT_READ == 0)
            {
                return KEY2_Data;
            }
        }
    }
    else if (KEY1_INPUT_READ == 0 && KEY2_INPUT_READ == 1)
    {
        /* Key release detected */
        s_key_up = 1;
        s_key_time = 0;
    }

    return KEY_None;

}

/******************************************************************************
  * @brief  DMA_RxConfig program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void DMA_RxConfig(void)
{
    DMA_InitTypeDef DMA_InitStructure;

    /* DMA Clocks enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

    /* Configure DMA byte transimit, direction: USART1 to SRAM */
    DMA_InitStructure.DMA_BufferSize = RX_SIZE;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)&g_cUsartRxBuff[0];
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&USART1->RDR;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_Priority = DMA_Priority_Low;
    DMA_Init(DMA1_Channel3, &DMA_InitStructure);

}

/******************************************************************************
  * @brief  fputc   program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
int fputc(int ch, FILE *f)
{
    USART_SendData(USART1, (uint8_t) ch);
    while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
    {

    }
  //  USART_SendData(USART1, (uint8_t) ch);
    return ch;
}
/**
  * @}
  */

#ifndef _KEIL_PRINTF_
#include <LowLevelIOInterface.h>

#pragma module_name = "?__write"

int MyLowLevelPutchar(int x)
{
  USART_SendData(USART1, (uint8_t) x);
    while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
    {

    }
  
  return x;
 
}

/*
 * If the __write implementation uses internal buffering, uncomment
 * the following line to ensure that we are called with "buffer" as 0
 * (i.e. flush) when the application terminates.
 */

size_t __write(int handle, const unsigned char * buffer, size_t size)
{
  /* Remove the #if #endif pair to enable the implementation */
#if 1

  size_t nChars = 0;

  if (buffer == 0)
  {
    /*
     * This means that we should flush internal buffers.  Since we
     * don't we just return.  (Remember, "handle" == -1 means that all
     * handles should be flushed.)
     */
    return 0;
  }

  /* This template only writes to "standard out" and "standard err",
   * for all other file handles it returns failure. */
  if (handle != _LLIO_STDOUT && handle != _LLIO_STDERR)
  {
    return _LLIO_ERROR;
  }

  for (/* Empty */; size != 0; --size)
  {
    if (MyLowLevelPutchar(*buffer++) < 0)
    {
      return _LLIO_ERROR;
    }

    ++nChars;
  }

  return nChars;

#else

  /* Always return error code when implementation is disabled. */
  return _LLIO_ERROR;

#endif
}

#endif
/************************ (C) COPYRIGHT FMD *****END OF FILE****/
