/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_rtc.c
* @author    : FMD AE
* @brief     : bsp_rtc program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_rtc.h"
#include "bsp_led.h"

#define LSE //extertal crystal oscillator 32.768KHz

extern uint16_t LedCnt;



extern uint32_t CaptureNumber, PeriodValue;
extern uint32_t IC1ReadValue1, IC1ReadValue2;

/**
  * @brief  delay time function.
  * @param  cnt  delay cnt ms
  * @retval None
  */
void DelaymS(uint16_t cnt)
{
    LedCnt = 0;
    while (1)
    {
        if (LedCnt > cnt)
            break;
    }
}


/**
  * @brief  Configures TIM14 to measure the LSI oscillator frequency.
  * @param  None
  * @retval LSI Frequency
  */
static uint32_t GetLSIFrequency(void)
{
    NVIC_InitTypeDef   NVIC_InitStructure;
    TIM_ICInitTypeDef  TIM_ICInitStructure;
    RCC_ClocksTypeDef  RCC_ClockFreq;

    /* TIM14 configuration *******************************************************/
    /* Enable TIM14 clock */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);

    /* Reset TIM14 registers */
    TIM_DeInit(TIM14);

    /* Configure TIM14 prescaler */
    TIM_PrescalerConfig(TIM14, 0, TIM_PSCReloadMode_Immediate);

    /* Connect internally the TIM14_CH1 to the RTC clock output */
    TIM_RemapConfig(TIM14, TIM14_RTC_CLK);

    /* TIM14 configuration: Input Capture mode ---------------------
       The reference clock(LSE or external) is connected to TIM14 CH1
       The Rising edge is used as active edge,
       The TIM14 CCR1 is used to compute the frequency value
    ------------------------------------------------------------ */
    TIM_ICInitStructure.TIM_Channel     = TIM_Channel_1;
    TIM_ICInitStructure.TIM_ICPolarity  = TIM_ICPolarity_Rising;
    TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
    TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV8;
    TIM_ICInitStructure.TIM_ICFilter = 0x0;
    TIM_ICInit(TIM14, &TIM_ICInitStructure);

    /* Enable the TIM14 global Interrupt */
    NVIC_InitStructure.NVIC_IRQChannel = TIM14_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    /* Enable TIM14 counter */
    TIM_Cmd(TIM14, ENABLE);

    /* Reset the flags */
    TIM14->SR = 0;

    /* Enable the CC1 Interrupt Request */
    TIM_ITConfig(TIM14, TIM_IT_CC1, ENABLE);


    /* Wait until the TIM14 get 2 LSI edges (refer to TIM14_IRQHandler() in
      stm32F0xx_it.c file) ******************************************************/
    while (CaptureNumber != 2)
    {
    }
    /* Deinitialize the TIM14 peripheral registers to their default reset values */
    TIM_DeInit(TIM14);

    /* Compute the LSI frequency, depending on TIM14 input clock frequency (PCLK1)*/
    /* Get SYSCLK, HCLK and PCLKx frequency */
    RCC_GetClocksFreq(&RCC_ClockFreq);

    /* PCLK1 prescaler equal to 1 => TIMCLK = PCLK1 */
    return ((RCC_ClockFreq.PCLK_Frequency / PeriodValue) * 8);
}


/**
  * @brief  Configures the RTC clock source.
  * @param  None
  * @retval None
  */
static void RTC_Config(void)
{
    RTC_InitTypeDef   RTC_InitStructure;
    RTC_TimeTypeDef   RTC_TimeStructure;
    uint32_t LSIFreq = 0;

    /* Enable the PWR clock */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

    /* Allow access to Backup Domain */
    PWR_BackupAccessCmd(ENABLE);

    /* Disable wake-up source(ALARM) to guarantee free access to WUT level-OR input */
    RTC_ITConfig(RTC_IT_ALRA, DISABLE);

    

    /* Enable wake-up source(ALARM) to guarantee free access to WUT level-OR input */
    RTC_ITConfig(RTC_IT_ALRA, ENABLE);

    /* Enable the LSI OSC */
    RCC_LSICmd(ENABLE);

    /* Wait till LSI is ready */
    while (RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET)
    {}
		/* Clear Wakeup flag */
    PWR_ClearFlag(PWR_FLAG_WU);
    /* Check if the StandBy flag is set */
    if (PWR_GetFlagStatus(PWR_FLAG_SB) != RESET)
    {
        /* Clear StandBy flag */
        PWR_ClearFlag(PWR_FLAG_SB);

        /* Check if the StandBy flag is cleared */
        if (PWR_GetFlagStatus(PWR_FLAG_SB) != RESET)
        {
            while (1);
        }

        RTC_WaitForSynchro();

        /* No need to configure the RTC as the RTC config(clock source, enable,
        prescaler,...) are kept after wake-up from STANDBY */
    }
    else
    {
        /* RTC Configuration ******************************************************/
        /* Reset Backup Domain */
        RCC_BackupResetCmd(ENABLE);
        RCC_BackupResetCmd(DISABLE);

        /* Select the RTC Clock Source */
        RCC_RTCCLKConfig(RCC_RTCCLKSource_LSI);

        /* Enable the RTC Clock */
        RCC_RTCCLKCmd(ENABLE);

        /* Wait for RTC APB registers synchronisation */
        RTC_WaitForSynchro();

        /* Get the LSI frequency:  TIM14 is used to measure the LSI frequency */
        LSIFreq = GetLSIFrequency();

        RTC_InitStructure.RTC_HourFormat = RTC_HourFormat_24;
        RTC_InitStructure.RTC_AsynchPrediv = 99;
        RTC_InitStructure.RTC_SynchPrediv = (LSIFreq / 100) - 1;

        RTC_Init(&RTC_InitStructure);

        /* Set the time to 01h 00mn 00s AM */
        RTC_TimeStructure.RTC_H12     = RTC_H12_AM;
        RTC_TimeStructure.RTC_Hours   = 0x01;
        RTC_TimeStructure.RTC_Minutes = 0x00;
        RTC_TimeStructure.RTC_Seconds = 0x00;

        RTC_SetTime(RTC_Format_BIN, &RTC_TimeStructure);
    }
}

/**
  * @brief  Enters STANDBY mode, RTC Alarm within 3 second or an external RESET
  *         will wake-up the system from STANDBY
  * @param  None
  * @retval None
  */
static void EnterSTANDBYMode(void)
{
    RTC_AlarmTypeDef  RTC_AlarmStructure;
    RTC_TimeTypeDef   RTC_TimeStructure;
		EXTI_InitTypeDef EXTI_InitStructure;
		NVIC_InitTypeDef NVIC_InitStructure;

    /* Disable the Alarm A */
    RTC_AlarmCmd(RTC_Alarm_A, DISABLE);

    /* Get the current time */
    RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);

    /* Set the alarm to current time + 3s */
    RTC_AlarmStructure.RTC_AlarmTime.RTC_H12     = RTC_TimeStructure.RTC_H12;
    RTC_AlarmStructure.RTC_AlarmTime.RTC_Hours   = RTC_TimeStructure.RTC_Hours;
    RTC_AlarmStructure.RTC_AlarmTime.RTC_Minutes = RTC_TimeStructure.RTC_Minutes;
    RTC_AlarmStructure.RTC_AlarmTime.RTC_Seconds = (RTC_TimeStructure.RTC_Seconds + 0x5) % 60;
    RTC_AlarmStructure.RTC_AlarmDateWeekDay = 31;
    RTC_AlarmStructure.RTC_AlarmDateWeekDaySel = RTC_AlarmDateWeekDaySel_Date;
    RTC_AlarmStructure.RTC_AlarmMask = RTC_AlarmMask_DateWeekDay | RTC_AlarmMask_Hours | RTC_AlarmMask_Minutes;
    RTC_SetAlarm(RTC_Format_BIN, RTC_Alarm_A, &RTC_AlarmStructure);

    /* Enable RTC Alarm A Interrupt: this Interrupt will wake-up the system from
    STANDBY mode (RTC Alarm IT not enabled in NVIC) */
    RTC_ITConfig(RTC_IT_ALRA, ENABLE);

    /* Enable the Alarm A */
    RTC_AlarmCmd(RTC_Alarm_A, ENABLE);

    /* Clear RTC Alarm Flag */
    RTC_ClearFlag(RTC_FLAG_ALRAF);
		
		
//		RTC_SetAlarm(RTC_Format_BIN, RTC_Alarm_A, &RTC_AlarmStructure);

//    RTC_ITConfig(RTC_IT_ALRA, ENABLE);

//    RTC_AlarmCmd(RTC_Alarm_A, ENABLE);
    RTC_ClearITPendingBit(RTC_IT_ALRA);
//		PWR_ClearFlag(PWR_FLAG_WU);
		
		
		
//		EXTI_ClearITPendingBit(EXTI_Line17);
//    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
//    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
//    EXTI_InitStructure.EXTI_Line = EXTI_Line17;
//    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
//    EXTI_Init(&EXTI_InitStructure);

//    /* NVIC configuration */
//    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
//    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
//    NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
//    NVIC_Init(&NVIC_InitStructure);
		

    /* Request to enter STANDBY mode (Wake Up flag is cleared in PWR_EnterSTANDBYMode function) */
    PWR_EnterSTANDBYMode();
}




void Bsp_PWR_Standby_Test(void)
{

    SETLED4(LED_ON);
    DelaymS(500);
		DelaymS(500);

    /* Configure RTC clock source and prescaler */
    RTC_Config();

    /* Enter STANDBY mode, RTC Alarm within 3 second or an external RESET will
     wake-up the system from STANDBY */
    SETLED4(LED_OFF);
		DelaymS(500);
		SETLED4(LED_ON);
		DelaymS(500);
    EnterSTANDBYMode();
//	NVIC_SystemReset(); 
    while (1)
    {

    }
}












