/**
  *********************************************************************************
  * @file           usb_core.c
  * @author         FMD AE
  * @brief          This file contains  functions for the USB firmware
  * @version        V1.0.0
  * @data           2022-01-13
  *********************************************************************************
  * @attention
  * COPYRIGHT (C) 2021 Fremont Micro Devices Corporation All rights reserved.
  *    This software is provided by the copyright holders and contributors,and the
  *software is believed to be accurate and reliable. However, Fremont Micro Devices
  *Corporation assumes no responsibility for the consequences of use of such
  *software or for any infringement of patents of other rights of third parties,
  *which may result from its use. No license is granted by implication or otherwise
  *under any patent rights of Fremont Micro Devices Corporation.
  *  ******************************************************************************
  */

/* Includes ----------------------------------------------------------------------*/
#include "usb_core.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/

/* Private variables -------------------------------------------------------------*/

/* Public variables --------------------------------------------------------------*/

/* Private function prototypes ---------------------------------------------------*/

/* Public function ------ --------------------------------------------------------*/

/******************************************************************************
  * @brief  USB_ReadRegister    program
  * @param  USB_Refister
  * @note
  * @retval register value
  *****************************************************************************
*/
uint8_t USB_ReadRegister(uint8_t USB_Refister)
{
    __IO uint32_t tmp = 0;

    /* Check the parameters */
    assert_param(IS_USB_REGISTER(USB_Refister));

    tmp = (uint32_t)USB;
    tmp += USB_Refister;

    /* Return the selected register value */
    return (*(__IO uint8_t *) tmp);
}
/******************************************************************************
  * @brief  USB_PDCTRLConfig    program
  * @param  PDCT
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_PDCTRLConfig(uint8_t PDCT)
{
    uint8_t temp = 0;

    assert_param(IS_USB_PDCTRL(PDCT));

    temp = USB->PDCTRL;
    temp &= ~USB_PDCTRL_FLAG;
    temp |= PDCT;
    USB->PDCTRL = temp;

}
/******************************************************************************
  * @brief  FIFORead    program
  * @param  nEP     Endpoint
  *         pDst    Read data buff
  * @note
  * @retval None
  *****************************************************************************
*/
void FIFORead(int nEP, int nBytes, void *pDst)
{
    int     nCount;
    BYTE   *pby;
    int     nAddr;

    if (nBytes)
    {
        nAddr = M_FIFO_EP0 + (nEP << 2);
        nCount = nBytes;
        pby = (BYTE *)pDst;
        while (nCount)
        {
            *pby++ = *((BYTE *)nAddr);
            nCount--;
        }
    }
}

/******************************************************************************
  * @brief  FIFOWrite program
  * @param  nEP     Endpoint
  *         nBytes  Number byte of data;
  *         pSrc    Write data buff
  * @note
  * @retval None
  *****************************************************************************
*/
void FIFOWrite(int nEP, int nBytes, void *pSrc)
{
    int     nCount;
    BYTE   *pby;
    int     nAddr;

    if (nBytes)
    {
        nAddr = M_FIFO_EP0 + (nEP << 2);
        nCount = nBytes;
        pby = (BYTE *)pSrc;
        while (nCount)
        {
            *((BYTE *)nAddr) = *pby++;
            nCount--;
        }
    }

}
/******************************************************************************
  * @brief  USB_ITConfig program
  * @param  USB_IT
  *         NewState
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_ITConfig(uint32_t USB_IT, FunctionalState NewState)
{
    __IO uint8_t *tmp = (__IO uint8_t *)USB_BASE;
    uint32_t IT_FLAG = 0;

    assert_param(IS_USB_CONFIG_IT(USB_IT));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (USB_IT & 0x00000100)
    {
        tmp = &USB->INTRINE;
        IT_FLAG = USB_IT & 0xfffffeff;
    }
    else if (USB_IT & 0x00000200)
    {
        tmp = &USB->INTROUTE;
        IT_FLAG = USB_IT & 0xfffffdff;
    }
    else if (USB_IT & 0x00000400)
    {
        tmp = &USB->INTRUSBE;
        IT_FLAG = USB_IT & 0xfffffbff;
    }

    if (NewState != DISABLE)
    {
        *((__IO uint8_t *)tmp) |= (uint8_t)IT_FLAG;
    }
    else
    {
        *((__IO uint8_t *)tmp) &= ~((uint8_t)IT_FLAG);
    }
}
/******************************************************************************
  * @brief  USB_GetITStatus program
  * @param  USB_IT
  * @note
  * @retval ITStatus
  *****************************************************************************
*/
ITStatus USB_GetITStatus(uint32_t USB_IT)
{
    __IO uint8_t *tmp = (__IO uint8_t *)USB_BASE;
    uint8_t tmp_flag = 0;
    uint32_t IT_FLAG = 0;
    ITStatus bitstatus = RESET;

    assert_param(IS_USB_CONFIG_IT(USB_IT));

    if (USB_IT & 0x00000100)
    {
        tmp = &USB->INTRIN;
        IT_FLAG = USB_IT & 0xfffffeff;
    }
    else if (USB_IT & 0x00000200)
    {
        tmp = &USB->INTROUT;
        IT_FLAG = USB_IT & 0xfffffdff;
    }
    else if (USB_IT & 0x00000400)
    {
        tmp = &USB->INTRUSB;
        IT_FLAG = USB_IT & 0xfffffbff;
    }

    tmp_flag = *(__IO uint8_t *)tmp;
    if ((tmp_flag & ((uint8_t)IT_FLAG)) != ((uint8_t)RESET))
    {
        bitstatus = SET;
    }
    else
    {
        bitstatus = RESET;
    }

    return bitstatus;
}


/*********************** (C) COPYRIGHT FMD ********************END OF FILE****/
