/**
  *********************************************************************************
  * @file           usb_ep.c
  * @author         FMD AE
  * @brief          This file contains  functions  for the usb_ep firmware
  *                 library.
  * @version        V1.0.0
  * @data           2022-01-13
  *********************************************************************************
  * @attention
  * COPYRIGHT (C) 2021 Fremont Micro Devices Corporation All rights reserved.
  *    This software is provided by the copyright holders and contributors,and the
  *software is believed to be accurate and reliable. However, Fremont Micro Devices
  *Corporation assumes no responsibility for the consequences of use of such
  *software or for any infringement of patents of other rights of third parties,
  *which may result from its use. No license is granted by implication or otherwise
  *under any patent rights of Fremont Micro Devices Corporation.
  *  ******************************************************************************
  */

/* Includes ----------------------------------------------------------------------*/
#include "usb_ep.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ---------------------------------------------------------------*/

/******************************************************************************
  * @brief  USB_Ep_Direc program
  * @param  ep
  *         Direction
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Ep_Direc(uint8_t ep, uint8_t Direction)
{
    /*set ep*/
    USB->INDEX = ep;
    /*clear direction*/
    *((BYTE *)M_REG_INCSR2) &= ~(0x01 << 5);
    /*set direction*/
    *((BYTE *)M_REG_INCSR2) |= Direction;
}
/******************************************************************************
  * @brief  Endpoint_Tx_data program
  * @param  pbistate
  * @note
  * @retval None
  *****************************************************************************
*/
void Endpoint_Tx_data(PM_EPBIN_STATUS pbistate)
{
    int     nBytes;
    BYTE    byInCSR;

    /* Check whether there is any data to send */
    if (pbistate->nBytesLeft != M_EP_NODATA)
    {
        /* Determine number of bytes to send */
        if (pbistate->nBytesLeft <= M_EP_MAXP)
        {
            nBytes = pbistate->nBytesLeft;
            pbistate->nBytesLeft = M_EP_NODATA;
        }
        else
        {
            nBytes = M_EP_MAXP;
            pbistate->nBytesLeft -= M_EP_MAXP;
        }

        /* Load FIFO */
        FIFOWrite((int)pbistate->byEP, nBytes, pbistate->pData);
        pbistate->pData = (BYTE *)pbistate->pData + nBytes;

        MWRITE_BYTE(M_REG_INDEX, pbistate->byEP);
        /*表示要发送的数据已经装入相应端点的FIFO中*/
        byInCSR = M_INCSR_IPR;
        MWRITE_BYTE(M_REG_INCSR1, byInCSR);
    }
}

/******************************************************************************
  * @brief  Endpoint_Rx_Data program
  * @param  pbistate
  * @note
  * @retval None
  *****************************************************************************
*/
void Endpoint_Rx_Data(PM_EPBOUT_STATUS pbostate)
{
    int     nBytes;
    BYTE    byOutCSR;

    /* Set index register */
    MWRITE_BYTE(M_REG_INDEX, pbostate->byEP);

    /* Read OUT CSR register */
    byOutCSR = MREAD_BYTE(M_REG_OUTCSR1);

    /* Get packet, */
    /* may need to unload more than one packet if double buffering enabled */
    /*端点FIFO 中有数据*/
    while (byOutCSR & M_OUTCSR_OPR)
    {

        /* Get packet size */
        nBytes = (int)MREAD_BYTE(M_REG_OUTCOUNT2);
        nBytes <<= 8;
        nBytes |= (int)MREAD_BYTE(M_REG_OUTCOUNT1);
        /* Check there is room in the buffer */
        if ((pbostate->nBytesRecv + nBytes) > pbostate ->nBuffLen)
        {
            USB_Rx_NAK(pbostate, ENABLE);
            return;
        }
        /* Unload FIFO */
        FIFORead((int)pbostate->byEP, nBytes, pbostate->pData);

        /* Update status */
        pbostate->nBytesRecv += nBytes;
        pbostate->pData = (BYTE *)pbostate->pData + nBytes;

        /* Check for end of transfer */
        if (nBytes <= M_EP_MAXP)
        {
            /* Call function to handle received data */

        }

        /* Clear OutPktRdy */
        MWRITE_BYTE(M_REG_OUTCSR1, 0);

        /* Check for another packet */
        byOutCSR = MREAD_BYTE(M_REG_OUTCSR1);
    }
}
/******************************************************************************
  * @brief  USB_Rx_NAK program
  * @param  pbistate
  * @note   
  * @retval None
  *****************************************************************************
*/
void USB_Rx_NAK(PM_EPBOUT_STATUS pbostate, FunctionalState NewState)
{
    uint32_t USB_It_Out_EpN = 0;
    switch (pbostate->byEP)
    {
    case  EP1:
        USB_It_Out_EpN = USB_IT_OUT_EP1_FLAG;
        break;
    case  EP2:
        USB_It_Out_EpN = USB_IT_OUT_EP2_FLAG;
        break;
    case  EP3:
        USB_It_Out_EpN = USB_IT_OUT_EP3_FLAG;
        break;
    case  EP4:
        USB_It_Out_EpN = USB_IT_OUT_EP4_FLAG;
        break;
    case  EP5:
        USB_It_Out_EpN = USB_IT_OUT_EP5_FLAG;
        break;
    case  EP6:
        USB_It_Out_EpN = USB_IT_OUT_EP6_FLAG;
        break;
    case  EP7:
        USB_It_Out_EpN = USB_IT_OUT_EP7_FLAG;
        break;
    default:
        break;
    }

    if (NewState != DISABLE)
    {

        USB_ITConfig(USB_It_Out_EpN, DISABLE);
    }
    else
    {
        USB_ITConfig(USB_It_Out_EpN, ENABLE);
        EndpointBulkOut(pbostate, M_EP_NORMAL);
    }
}
/******************************************************************************
  * @brief  EndpointBulkIn  program
  * @param  pbistate
  *         nCallState
  * @note   EndpointBulkIn service routine
  *         This routine provides simple handling for bulk in endpoints.
  *         Endpoint MaxP is defined in endpoint.h.
  *         While the nBytesLeft field of the pbostate structure is >= 0
  *         the endpoint will send packets of MaxP, with the residue in the last packet.
  *         If the data to send is an exact multiple of MaxP then a null data packet
  *         will be sent.
  * @retval None
  *****************************************************************************
*/
void EndpointBulkIn(PM_EPBIN_STATUS pbistate, int nCallState)
{
    BYTE    byInCSR;

    /* Reset endpoint */
    if (nCallState == M_EP_RESET)
    {
        /*切换端点号*/
        MWRITE_BYTE(M_REG_INDEX, pbistate->byEP);
        /*清除相应端点的数据翻转状态和清空FIFO中要发送的数据*/
        byInCSR = M_INCSR_CDT | M_INCSR_FF;
        MWRITE_BYTE(M_REG_INCSR1, byInCSR);
    }
    else
    {
        Endpoint_Tx_data(pbistate);
    }
}



/******************************************************************************
  * @brief  EndpointBulkOut program
  * @param  pbistate
  *         nCallState
  * @note   EndpointBulkOut service routine
  *         This routine provides simple handling for bulk out endpoints.
  *         Endpoint MaxP is defined in endpoint.h.
  *         It assumes a transfer is complete when a packet of less than MaxP is received.
  * @retval None
  *****************************************************************************
*/
void EndpointBulkOut(PM_EPBOUT_STATUS pbostate, int nCallState)
{
    BYTE    byOutCSR;

    /* Reset endpoint */
    if (nCallState == M_EP_RESET)
    {
        /***配置端点号***/
        MWRITE_BYTE(M_REG_INDEX, pbostate->byEP);
        byOutCSR = M_OUTCSR_CDT | M_OUTCSR_FF;
        /***复位端点的数据反转状态,清空接收 FIFO 中的数据***/
        MWRITE_BYTE(M_REG_OUTCSR1, byOutCSR);
    }
    else
    {
        Endpoint_Rx_Data(pbostate);
    }
}

/************************** (C) COPYRIGHT FMD **********************END OF FILE****/

