/**
  *********************************************************************************
  * @file           usb_ep0.c
  * @author         FMD AE
  * @brief          This file contains  functions  for the usb_ep0 firmware
  *                 library.
  * @version        V1.0.0
  * @data           2022-01-13
  *********************************************************************************
  * @attention
  * COPYRIGHT (C) 2021 Fremont Micro Devices Corporation All rights reserved.
  *    This software is provided by the copyright holders and contributors,and the
  *software is believed to be accurate and reliable. However, Fremont Micro Devices
  *Corporation assumes no responsibility for the consequences of use of such
  *software or for any infringement of patents of other rights of third parties,
  *which may result from its use. No license is granted by implication or otherwise
  *under any patent rights of Fremont Micro Devices Corporation.
  *  ******************************************************************************
  */

/* Includes ----------------------------------------------------------------------*/
#include "usb_ep0.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/

/* Private define ----------------------------------------------------------------*/

/* Private variables -------------------------------------------------------------*/
uint8_t         INCSRDATA;
/* Pointer to current configuration descriptor */
void *gpCurCfg = NULL;

uint8_t USB_CBW_STATUS = CBW_CMD_STAGE;
/* Pointer to current interface descriptors */
PSTD_IF_DSCR    gpCurIf[M_MAXIFS];
/* Current alternate interface values */
BYTE    gbyCurIfVal[M_MAXIFS];
CLASS_HID_DATA  c_hid_data = {0};


DEVICE_INFO Device_Info;

DEVICE_INFO *pInformation = &Device_Info;
uint8_t Max_Lun[15] = {0};
/* Public variables --------------------------------------------------------------*/
static uint8_t g_port_status = 0x18U;
extern uint8_t Printer_DEVICE_ID[DEVICE_ID_LEN];
/* Current device state */
int gnDevState = DEVSTATE_DEFAULT;
/* Private function prototypes ---------------------------------------------------*/
static void USB_Endpoint0_Command(PM_EP0_STATUS, PCOMMAND);
static void USB_Endpoint0_Tx(PM_EP0_STATUS);
static void USB_StdDev_Req(PM_EP0_STATUS, PCOMMAND);
static int ConfigureIfs(void);
static void USB_Endpoint0_Rx(PM_EP0_STATUS pep0state);
static void USB_Class_Req(PM_EP0_STATUS pep0state, PCOMMAND pcmd);

/* Public function ------ --------------------------------------------------------*/
/******************************************************************************
  * @brief  USB_Endpoint0   program
  * @param  nCallState
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Endpoint0(int nCallState)
{
    static M_EP0_STATUS ep0state;
    static COMMAND  cmd;
    BYTE    byCSR0;

    /* Check for USB reset of endpoint 0 */
    if (nCallState == M_EP_RESET)
    {
        ep0state.nState = M_EP0_IDLE;
        ep0state.byFAddr = 0xFF;
        /* Clear current configuration pointer */
        gpCurCfg = NULL;
        return;
    }

    /* Read CSR0 */
    MWRITE_BYTE(M_REG_INDEX, 0);
    byCSR0 = MREAD_BYTE(M_REG_CSR0);

    /* Check for status stage of a request */
    /*接收已完成*/
    if (!(byCSR0 & M_CSR0_OUTPKTRDY))
    {
        /* Complete SET_ADDRESS command */
        if (ep0state.byFAddr != 0xFF)
        {
            MWRITE_BYTE(M_REG_FADDR, ep0state.byFAddr);
            if ((gnDevState == DEVSTATE_DEFAULT) && ep0state.byFAddr)
            {
                gnDevState = DEVSTATE_ADDRESS;
            }
            else if ((gnDevState == DEVSTATE_ADDRESS) && !ep0state.byFAddr)
            {
                gnDevState = DEVSTATE_DEFAULT;
            }
        }
    }
    /* Clear pending commands */
    ep0state.byFAddr = 0xFF;

    /* Check for SentStall */
    if (byCSR0 & M_CSR0_SENTSTALL)
    {
        /*清除STALL 位*/
        MWRITE_BYTE(M_REG_CSR0, (byCSR0 & ~M_CSR0_SENDSTALL));
        ep0state.nState = M_EP0_IDLE;
    }

    /* Check for SetupEnd */
    if (byCSR0 & M_CSR0_SETUPEND)
    {
        /*清除setupend位*/
        MWRITE_BYTE(M_REG_CSR0, (byCSR0 | M_CSR0_SVDSETUPEND));
        ep0state.nState = M_EP0_IDLE;
    }

    /* Call relevant routines for endpoint 0 state */
    if (ep0state.nState == M_EP0_IDLE)
    {
        /* If no packet has been received, */
        /* assume that this was a STATUS phase complete. */
        /* Otherwise load new command */
        if (byCSR0 & M_CSR0_OUTPKTRDY)
        {
            /* Read the 8-byte command from the FIFO */
            /* There is no need to check that OutCount is set to 8 */
            /* as the MUSBFSFC will reject SETUP packets that are not 8 bytes long. */
            FIFORead(0, 8, &cmd);
            USB_Endpoint0_Command(&ep0state, &cmd);
        }
    }

    if (ep0state.nState == M_EP0_TX)
    {
        USB_Endpoint0_Tx(&ep0state);
    }
    else if (ep0state.nState == M_EP0_RX)
    {
        USB_Endpoint0_Rx(&ep0state);
    }
}

/******************************************************************************
  * @brief  HID_SetReportEvent  program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
void HID_SetReportEvent(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    if ((pcmd->bmRequestType == HID_REQUEST_SET) && (pcmd->USBwIndex == 0))// Interface 0
    {
        switch (pcmd->USBwValue >> 8)
        {
        case    HID_REPORT_INPUT:
            break;
        case    HID_REPORT_OUTPUT:
            /*保存请求类型*/
            c_hid_data.wHID_Status = HID_REQUEST_SET_REPORT;

            pep0state->nBytesLeft = pcmd->USBwLength;
            pep0state->nState = M_EP0_RX;
            pep0state->pData = (void *)&c_hid_data.wHID_SetRptByte[0];
            MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
            break;
        case    HID_REPORT_FEATURE:
            break;
        }
    }
}
/******************************************************************************
  * @brief  HID_GetReportEvent  program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
void HID_GetReportEvent(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    if (gnDevState > DEVSTATE_ADDRESS)
    {
        switch (pcmd->USBwValue >> 8)
        {
        case    HID_REPORT_INPUT:
        case    HID_REPORT_FEATURE:
        case    HID_REPORT_OUTPUT:
            /*保存请求类型*/
            c_hid_data.wHID_Status = HID_REQUEST_GET_REPORT;

            pep0state->nBytesLeft = 0x01;
            pep0state->pData = (void *)&c_hid_data.wHID_SetRptByte[0];
            pep0state->nState = M_EP0_TX;
            MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
            break;
        }
    }
}
/******************************************************************************
  * @brief  HID_GetIdleEvent    program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
void HID_GetIdleEvent(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    /*保存请求类型*/
    c_hid_data.wHID_Status = HID_REQUEST_GET_IDLE;

    pep0state->nBytesLeft = 0x01;
    pep0state->pData = (void *)&c_hid_data.wHID_IdleTimeIf0ID;
    pep0state->nState = M_EP0_TX;
    MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
}
/******************************************************************************
  * @brief  HID_SetIdleEvent    program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
void HID_SetIdleEvent(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    /*保存请求类型*/
    c_hid_data.wHID_Status = HID_REQUEST_SET_IDLE;

    pep0state->nState = M_EP0_IDLE;
    c_hid_data.wHID_IdleTimeIf0ID = (pcmd->USBwValue >> 8);
    MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
}

/******************************************************************************
  * @brief  HID_GetProtocolEvent    program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
void HID_GetProtocolEvent(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    /*保存请求类型*/
    c_hid_data.wHID_Status = HID_REQUEST_GET_PROTOCOL;

    pep0state->nBytesLeft = 0x01;
    pep0state->pData = (void *)&c_hid_data.wHID_Protocol;
    pep0state->nState = M_EP0_TX;
    MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
}
/******************************************************************************
  * @brief  HID_SetProtocolEvent    program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
void HID_SetProtocolEvent(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    /*保存请求类型*/
    c_hid_data.wHID_Status = HID_REQUEST_SET_PROTOCOL;

    pep0state->nState = M_EP0_IDLE;
    c_hid_data.wHID_Protocol = (pcmd->USBwValue & 0x00ff);
    MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
}
/******************************************************************************
  * @brief  USB_HID_Class_Req   program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_HID_Class_Req(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    switch (pcmd->bRequest)
    {
    case    HID_REQUEST_GET_REPORT:
    {
        HID_GetReportEvent(pep0state, pcmd);
        return;
    }
    case    HID_REQUEST_GET_IDLE:
    {
        HID_GetIdleEvent(pep0state, pcmd);
        return;
    }
    case    HID_REQUEST_GET_PROTOCOL:
    {
        HID_GetProtocolEvent(pep0state, pcmd);
        return;
    }
    case    HID_REQUEST_SET_REPORT:
    {
        HID_SetReportEvent(pep0state, pcmd);
        return;
    }
    case    HID_REQUEST_SET_IDLE:
    {
        HID_SetIdleEvent(pep0state, pcmd);
        return;
    }
    case    HID_REQUEST_SET_PROTOCOL:
    {
        HID_SetProtocolEvent(pep0state, pcmd);
        return;
    }
    default:
    {
        break;
    }
    }
    MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY | M_CSR0_SENDSTALL);
}

/******************************************************************************
  * @brief  USB_MAX_LUN program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_MAX_LUN(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    switch (pcmd->bRequest)
    {
    //0xA1
    case    GET_MAX_LUN:                    //0XFE
        /*获取最大逻辑单元*/
        //printf("获取最大逻辑单元。\r\n");
        pep0state->nBytesLeft = 0x01;
        pep0state->pData = (void *)&Max_Lun;
        pep0state->nState = M_EP0_TX;
        MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
        return;
    //0x21
    case    MASS_STORAGE_RESET:                 //0XFF
        /*大容量存储设备复位*/
        //printf("大容量存储设备复位。\r\n");
        pep0state->nState = M_EP0_IDLE;
        MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);

        /*接下来的状态是命令阶段(CBW)*/
        USB_CBW_STATUS = CBW_CMD_STAGE;
        return;
    }
    MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY | M_CSR0_SENDSTALL);
}



/******************************************************************************
  * @brief  USB_Endpoint0_Command   program
  * @param  pep0state
  *         pcmd
  * @note
  * @retval None
  *****************************************************************************
*/
static void USB_Endpoint0_Command(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    /* Check request type */
    switch (pcmd->bmRequestType & M_CMD_TYPEMASK)
    {
    /***主机到设备，标准请求命令，接收者为设备***/
    case M_CMD_STDREQ:
        USB_StdDev_Req(pep0state, pcmd);
        break;

    /***主机到设备，类请求命令，接收者为设备***/
    case M_CMD_CLASSREQ:
        /***主机到设备，用户定义的命令，接收者为设备***/
        USB_Class_Req(pep0state, pcmd);
        break;
    case M_CMD_VENDREQ:

        break;
    /***保留***/
    default:
        /* Stall the command if a reserved request is received */
        MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY | M_CSR0_SENDSTALL);
        break;
    }

    return;
}
/******************************************************************************
  * @brief  USB_Class_Req  program
  * @param  pep0state
  *         pcmd
  * @note   Standard Device Request
  *         This routine is called when a standard device request has been received.
  *         The bRequest field of the received command is decoded.
  * @retval None
  *****************************************************************************
*/


static void USB_Class_Req(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    int bError = FALSE;
    BYTE    by;
    switch (pcmd->bRequest)
    {
    case GET_DEVICE_ID:
        pep0state->nBytesLeft = sizeof(Printer_DEVICE_ID);;
        if (pcmd->USBwLength < pep0state->nBytesLeft)
        {
            pep0state->nBytesLeft = pcmd->USBwLength;
        }

        pep0state->pData = (void *)Printer_DEVICE_ID;
        pep0state->nState = M_EP0_TX;
        break;
    case GET_PORT_STATUS:
        
        pep0state->nBytesLeft = 1;
        if (pcmd->USBwLength < pep0state->nBytesLeft)
        {
            pep0state->nBytesLeft = pcmd->USBwLength;
        }
        pep0state->pData = (uint8_t *)&g_port_status;
        pep0state->nState = M_EP0_TX;        
        break;
    case SOFT_RESET:

        break;    
    default:
        bError = TRUE;
    }
    /* Stall command if an error occurred */
    if (bError)
    {
        /***清零 OUTPKTRDY 位,终止当前传输***/
        by = M_CSR0_SVDOUTPKTRDY | M_CSR0_SENDSTALL;
    }
    MWRITE_BYTE(M_REG_CSR0, by);
}

/******************************************************************************
  * @brief  USB_StdDev_Req  program
  * @param  pep0state
  *         pcmd
  * @note   Standard Device Request
  *         This routine is called when a standard device request has been received.
  *         The bRequest field of the received command is decoded.
  * @retval None
  *****************************************************************************
*/

static void USB_StdDev_Req(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
    int bError = FALSE, bNoData = FALSE, n;
    BYTE    by;
    BYTE    byConfig;
    PSTD_CFG_DSCR   pcfg;

    uint8_t Related_Endpoint;
    uint8_t wIndex0 = pcmd->USBwIndex0;
    Related_Endpoint = (wIndex0 & 0x0f);
    uint8_t srbuff;
    uint8_t txdat[2] = {0};
    switch (pcmd->bRequest)
    {
    /***用来给设备分配地址***/
    case SET_ADDRESS:
        /* Store device function address until status stage of request */
        if (pcmd->bmRequestType != M_CMD_STDDEVOUT)
        {
            bError = TRUE;
        }
        else if (gnDevState <= DEVSTATE_ADDRESS)
        {
            pep0state->byFAddr = (BYTE)pcmd->USBwValue;
            bNoData = TRUE;
        }
        else
        {
            bError = TRUE;
        }
        break;

    /***用于主机获取设备的特定描述符***/
    case GET_DESCRIPTOR:
        /* Decode the required descriptor from the command */
        switch ((pcmd->bmRequestType & USB_RECPTYPE_MASK))
        {
        case USB_RECPTYPE_DEV:
            /***报告描述符的接受者是接口，pcmd->bmRequestType = 0x81时传输HID报告描述符***/
            if (!(pcmd->bmRequestType & M_CMD_STDDEVIN))
            {
                bError = TRUE;
            }
            else
            {
                switch (pcmd->USBwValue & M_CMD_DESCMASK)
                {
                /***设备描述符***/
                case M_CMD_DEVICE:
                    /* Prepare to return Standard Device Descriptor */
                    pep0state->nBytesLeft = USB_FSDeviceDescriptorSize;
                    /* Check host is allowing a descriptor this long */
                    if (pcmd->USBwLength < pep0state->nBytesLeft)
                    {
                        pep0state->nBytesLeft = pcmd->USBwLength;
                    }

                    pep0state->pData = (void *)USB_FSDeviceDescriptor;
                    pep0state->nState = M_EP0_TX;
                    break;
                /***设备限定描述符***/
                case M_CMD_DEVQUAL:
                    /* No alternate speed supported */
                    bError = TRUE;
                    break;
                /***配置描述符***/
                case M_CMD_CONFIG:
                    byConfig = (BYTE)(pcmd->USBwValue & 0x00FF);
                    if (byConfig > USB_FSDeviceDescriptor[USB_FSDeviceDescriptorSize - 1])
                    {
                        bError = TRUE;
                    }
                    else
                    {
                        /* Get pointer to requested configuration descriptor */
                        /* Prepare to return Configuration Descriptors */
                        pep0state->nBytesLeft = USB_FSConfigDescriptorSize;
                        pep0state->pData = (void *)USB_FSConfigDescriptor;
                        pep0state->nState = M_EP0_TX;
                        /* Check host is allowing a descriptor this long */
                        if (pcmd->USBwLength < pep0state->nBytesLeft)
                        {
                            pep0state->nBytesLeft = pcmd->USBwLength;
                        }
                    }
                    break;
                /***其它速度配置描述符***/
                case M_CMD_OTHERSPEED:
                    /* No alternate speed supported */
                    bError = TRUE;
                    break;
                /***HID报告描述符***/
                case    M_CMD_HID_REPORT_DESC:      //报告描述符     0x2200
                    /* Prepare to return Standard Device Descriptor */
                    switch (pcmd->USBwIndex)
                    {
                    case    0:
                        //接口0
                        pep0state->nBytesLeft = USB_HID_FSReportDescriptor_1Size;       //报告描述符的长度
                        /* Check host is allowing a descriptor this long */
                        if (pcmd->USBwLength < pep0state->nBytesLeft)
                        {
                            pep0state->nBytesLeft = pcmd->USBwLength;
                        }
                        pep0state->pData = (void *)USB_HID_FSReportDescriptor_1;
                        pep0state->nState = M_EP0_TX;
                        break;
                    case    1:
                        //接口1
                        pep0state->nBytesLeft = USB_HID_FSReportDescriptor_2Size;       //报告描述符的长度
                        /* Check host is allowing a descriptor this long */
                        if (pcmd->USBwLength < pep0state->nBytesLeft)
                        {
                            pep0state->nBytesLeft = pcmd->USBwLength;
                        }
                        pep0state->pData = (void *)USB_HID_FSReportDescriptor_2;
                        pep0state->nState = M_EP0_TX;
                        break;
                    default:
                        //接口
                        bError = TRUE;
                        break;
                    }
                    break;
                case    M_CMD_STRING:   //字符串描述符
                    switch (pcmd->USBwValue & 0xff)
                    {
                    case    0:  //获取语言ID
                        pep0state->nBytesLeft = LanguageIdSize;
                        /* Check host is allowing a descriptor this long */
                        if (pcmd->USBwLength < pep0state->nBytesLeft)
                        {
                            pep0state->nBytesLeft = pcmd->USBwLength;
                        }
                        pep0state->pData = (void *)LanguageId;
                        pep0state->nState = M_EP0_TX;
                        break;
                    case    1:  //因厂商字符串索引值为1，所以这里写1
                        pep0state->nBytesLeft = ManufacturerStringDescriptorSize;
                        /* Check host is allowing a descriptor this long */
                        if (pcmd->USBwLength < pep0state->nBytesLeft)
                        {
                            pep0state->nBytesLeft = pcmd->USBwLength;
                        }
                        pep0state->pData = (void *)ManufacturerStringDescriptor;
                        pep0state->nState = M_EP0_TX;
                        break;
                    case    2:  //因产品字符串索引值为2，所以这里写2
                        pep0state->nBytesLeft = ProductStringDescriptorSize;
                        /* Check host is allowing a descriptor this long */
                        if (pcmd->USBwLength < pep0state->nBytesLeft)
                        {
                            pep0state->nBytesLeft = pcmd->USBwLength;
                        }
                        pep0state->pData = (void *)ProductStringDescriptor;
                        pep0state->nState = M_EP0_TX;
                        break;
                    case    3:  //因产品序列号索引值为3，所以这里写3
                        pep0state->nBytesLeft = SerialNumberSringDescriptorSize;
                        /* Check host is allowing a descriptor this long */
                        if (pcmd->USBwLength < pep0state->nBytesLeft)
                        {
                            pep0state->nBytesLeft = pcmd->USBwLength;
                        }
                        pep0state->pData = (void *)SerialNumberSringDescriptor;
                        pep0state->nState = M_EP0_TX;
                        break;
                    default:
                        bError = TRUE;
                        break;
                    }
                    break;
                /***保留***/
                default:
                    bError = TRUE;
                }
            }

            break;
        case USB_RECPTYPE_ITF:
            pep0state->nBytesLeft = 1U;
            if (pcmd->USBwLength < pep0state->nBytesLeft)
            {
                pep0state->nBytesLeft = pcmd->USBwLength;
            }

            pep0state->pData = (void *)(&g_port_status);
            pep0state->nState = M_EP0_TX;
            break;
        case USB_RECPTYPE_EP:
            break;

        default:
            break;
        }

        break;

    /***用于主机指示设备采用的要求的配置***/
    case SET_CONFIGURATION:
        byConfig = (BYTE)(pcmd->USBwValue & 0x00FF);
        if (gnDevState == DEVSTATE_DEFAULT)
        {
            bError = TRUE;
        }
        /* Assumes configurations are numbered 1 to NumConfigurations */
        else if (byConfig > USB_FSDeviceDescriptor[USB_FSDeviceDescriptorSize - 1])
        {
            bError = TRUE;
        }
        else if (!byConfig)
        {
            gnDevState = DEVSTATE_ADDRESS;
        }
        else
        {
            /* Get pointer to requested configuration descriptor */
            gpCurCfg = (void *)USB_FSConfigDescriptor;
            /* Set all alternate settings to zero */
            for (n = 0; n < M_MAXIFS; n++)
                gbyCurIfVal[n] = 0;
            /* Configure endpoints */
            ConfigureIfs();
            gnDevState = DEVSTATE_CONFIG;
            MWRITE_BYTE(M_REG_INDEX, 0);
            bNoData = TRUE;
        }
        break;

    /***用于主机要求设备用某个描述符来描述接口***/
    case SET_INTERFACE:
        if (gnDevState > DEVSTATE_ADDRESS)
        {
            n = (int)pcmd->USBwIndex;
            pcfg = (PSTD_CFG_DSCR)gpCurCfg;
            if ((BYTE)n >= pcfg->bNumInterfaces)
            {
                bError = TRUE;
            }
            else
            {
                gbyCurIfVal[n] = (BYTE)pcmd->USBwValue;
                if (ConfigureIfs() == FALSE)
                {
                    bError = TRUE;
                }

                MWRITE_BYTE(M_REG_INDEX, 0);
                bNoData = TRUE;
            }
        }
        else
        {
            bError = TRUE;
        }
        break;

    /***用于主机获取设备当前设备的配置值***/
    case GET_CONFIGURATION:
        if ((gnDevState == DEVSTATE_ADDRESS) && (!pcmd->USBwValue))
        {
            /* Prepare to return zero */
            pep0state->nBytesLeft = 1;
            pep0state->pData = (void *)&pcmd->USBwValue;
            pep0state->nState = M_EP0_TX;
        }
        else if (gnDevState > DEVSTATE_ADDRESS)
        {
            /* Prepare to return configuration value */
            pcfg = (PSTD_CFG_DSCR)gpCurCfg;
            pep0state->nBytesLeft = 1;
            pep0state->pData = (void *)&pcfg->bConfigurationValue;
            pep0state->nState = M_EP0_TX;
        }
        else
        {
            bError = TRUE;
        }
        break;

    /***用于获取当前某个接口描述符编号***/
    case GET_INTERFACE:
        if (gnDevState > DEVSTATE_ADDRESS)
        {
            n = (int)pcmd->USBwIndex;
            pcfg = (PSTD_CFG_DSCR)gpCurCfg;
            if ((BYTE)n >= pcfg->bNumInterfaces)
            {
                bError = TRUE;
            }
            else
            {
                /* Prepare to return interface value */
                pep0state->nBytesLeft = 1;
                pep0state->pData = (void *)&gpCurIf[n]->bInterfaceNumber;
                pep0state->nState = M_EP0_TX;
            }
        }
        else
            bError = TRUE;
        break;

    /***用来启用或激活命令接收者的某些特性***/
    case SET_FEATURE:
        switch (pcmd->bmRequestType)
        {
        case M_CMD_STDDEVOUT:
            switch (pcmd->USBwValue)
            {
            case M_FTR_DEVREMWAKE:
                pInformation->Current_Feature = pInformation->Current_Feature | (0x1 << 5);
                bNoData = TRUE;
                break;
            default:
                bError = TRUE;
                break;
            }
            break;

        case M_CMD_STDIFOUT:

        /* SET FEATURE for EndPoint*/
        case M_CMD_STDEPOUT:

            switch (pcmd->USBwValue0)
            {
            case M_FTR_EPHALT:

                MWRITE_BYTE(M_REG_INDEX, Related_Endpoint);
                if (pcmd->USBwIndex0 & 0x80)
                {
                    if (Related_Endpoint) /* IN endpoint */
                    {
                        srbuff = *((uint8_t *)M_REG_INCSR1);
                        srbuff = srbuff | (0x1 << 4);
                        MWRITE_BYTE(M_REG_INCSR1, srbuff);
                        INCSRDATA = MREAD_BYTE(M_REG_INCSR1);
                        __ASM("NOP");
                    }

                    else
                    {
                        srbuff = *((uint8_t *)M_REG_CSR0);
                        *((uint8_t *)M_REG_CSR0) = srbuff | (0x1 << 5);
                        srbuff = *((uint8_t *)M_REG_CSR0);
                    }

                }

                else
                {
                    /* OUT endpoint */
                    if (Related_Endpoint)
                    {
                        srbuff = *((uint8_t *)M_REG_OUTCSR1);
                        *((uint8_t *)M_REG_OUTCSR1) = srbuff | (0x1 << 5);
                        srbuff = *((uint8_t *)M_REG_OUTCSR1);
                    }

                    else
                    {
                        srbuff = *((uint8_t *)M_REG_CSR0);
                        *((uint8_t *)M_REG_CSR0) = srbuff | (0x1 << 5);
                        srbuff = *((uint8_t *)M_REG_CSR0);
                    }

                }
                bNoData = TRUE;
                MWRITE_BYTE(M_REG_INDEX, 0);
                break;
            default:
                bError = TRUE;
                break;
            }
            break;

        default:
            bError = TRUE;
            break;
        }
        break;

    /***用来CLEAR接收者的某些特性***/
    case CLEAR_FEATURE:
        switch (pcmd->bmRequestType)
        {
        case M_CMD_STDDEVOUT:
            switch (pcmd->USBwValue)
            {
            case M_FTR_DEVREMWAKE:
                pInformation->Current_Feature = pInformation->Current_Feature & (~(0x1 << 5));
                bNoData = TRUE;
                break;
            default:
                bError = TRUE;
                break;
            }
            break;

        case M_CMD_STDIFOUT:

        /* CLEAR FEATURE for EndPoint*/
        case M_CMD_STDEPOUT:
            switch (pcmd->USBwValue0)
            {
            case M_FTR_EPHALT:
                MWRITE_BYTE(M_REG_INDEX, Related_Endpoint);
                if (pcmd->USBwIndex0 & 0x80)
                {
                    /* IN endpoint */
                    if (Related_Endpoint)
                    {
                        //                                  *((uint8_t*)M_REG_INCSR1) = *((uint8_t*)M_REG_INCSR1) & (~(0x01<<6));
                        srbuff = *((uint8_t *)M_REG_INCSR1);
                        *((uint8_t *)M_REG_INCSR1) = srbuff & (~(0x01 << 6));
                        srbuff = *((uint8_t *)M_REG_INCSR1);

                        //                                  *((uint8_t*)M_REG_INCSR1) = *((uint8_t*)M_REG_INCSR1) & (~(0x01<<4));
                        srbuff = *((uint8_t *)M_REG_INCSR1);
                        *((uint8_t *)M_REG_INCSR1) = srbuff & (~(0x01 << 4));
                        INCSRDATA = *((uint8_t *)M_REG_INCSR1);
                        __ASM("NOP");
                    }

                    else
                    {
                        //                                  *((uint8_t*)M_REG_CSR0) = *((uint8_t*)M_REG_CSR0) & (~(0x01<<5));
                        srbuff = *((uint8_t *)M_REG_CSR0);
                        *((uint8_t *)M_REG_CSR0) = srbuff & (~(0x01 << 5));
                        srbuff = *((uint8_t *)M_REG_CSR0);

                    }

                }

                else
                {
                    /* OUT endpoint */
                    if (Related_Endpoint)
                    {
                        //                                  *((uint8_t*)M_REG_OUTCSR1) = *((uint8_t*)M_REG_OUTCSR1) & (~(0x01<<7));
                        //                                  *((uint8_t*)M_REG_OUTCSR1) = *((uint8_t*)M_REG_OUTCSR1) & (~(0x01<<5));
                        srbuff = *((uint8_t *)M_REG_OUTCSR1);
                        *((uint8_t *)M_REG_OUTCSR1) = srbuff & (~(0x01 << 7));
                        srbuff = *((uint8_t *)M_REG_OUTCSR1);

                        srbuff = *((uint8_t *)M_REG_OUTCSR1);
                        *((uint8_t *)M_REG_OUTCSR1) = srbuff & (~(0x01 << 5));
                        srbuff = *((uint8_t *)M_REG_OUTCSR1);

                    }

                    else
                    {
                        //                                  *((uint8_t*)M_REG_CSR0) = *((uint8_t*)M_REG_CSR0)  & (~(0x01<<5));
                        srbuff = *((uint8_t *)M_REG_CSR0);
                        *((uint8_t *)M_REG_CSR0) = srbuff & (~(0x01 << 5));
                        srbuff = *((uint8_t *)M_REG_CSR0);
                    }

                }
                bNoData = TRUE;
                MWRITE_BYTE(M_REG_INDEX, 0);
                break;

            default:
                bError = TRUE;
                break;
            }
            break;

        default:
            bError = TRUE;
            break;
        }
        break;


    /***用于主机获取设备 STATUS***/
    case GET_STATUS:

        switch (pcmd->bmRequestType)
        {

        /* GET STATUS for Device*/
        case M_CMD_STDDEVIN:

            if (pInformation->Current_Feature & (0x1 << 5))
            {
                txdat[0] |= (0x1 << 1);
            }
            else
            {
                txdat[0] &= ~(0x1 << 1);
            }
            if (pInformation->Current_Feature & (0x1 << 6))
            {
                txdat[0] |= (0x1 << 0);
            }
            else
            {
                txdat[0] &= ~(0x1 << 0);
            }

            txdat[1] = 0;
            pep0state->nBytesLeft = 2;
            pep0state->pData = txdat;
            pep0state->nState = M_EP0_TX;
            break;

        /* GET STATUS for Interface*/
        case M_CMD_STDIFIN:
            pep0state->nBytesLeft = 2;
            pep0state->pData = txdat;
            pep0state->nState = M_EP0_TX;
            break;
        /* GET STATUS for EndPoint*/
        case M_CMD_STDEPIN:

            if (pcmd->USBwIndex0 & 0x80)
            {
                if (Related_Endpoint)
                {
                    MWRITE_BYTE(M_REG_INDEX, Related_Endpoint);
                    INCSRDATA = MREAD_BYTE(M_REG_INCSR1);
                    if (INCSRDATA & (0x1 << 4))
                        txdat[0] = 1;
                    else txdat[0] = 0;
                }
                else if (*((uint8_t *)M_REG_CSR0) & (0x1 << 2))
                    txdat[0] = 1;
                else txdat[0] = 0;
            }

            else
            {
                if (Related_Endpoint)
                {
                    if (*((uint8_t *)M_REG_OUTCSR1) & (0x1 << 5))
                        txdat[0] = 1;
                    else txdat[0] = 0;
                }
                else if (*((uint8_t *)M_REG_CSR0) & (0x1 << 2))
                    txdat[0] = 1;
                else txdat[0] = 0;
            }
            txdat[1] = 0;
            pep0state->nBytesLeft = 2;
            pep0state->pData = txdat;
            pep0state->nState = M_EP0_TX;
            MWRITE_BYTE(M_REG_INDEX, 0);
            break;

        default:
            bError = TRUE;
            break;

        }
        break;


    /***不可识别的请求***/
    default:
        /* Stall the command if an unrecognized request is received */
        bError = TRUE;
        break;
    }



    /* Stall command if an error occurred */
    if (bError)
    {
        /***清零 OUTPKTRDY 位,终止当前传输***/
        by = M_CSR0_SVDOUTPKTRDY | M_CSR0_SENDSTALL;
    }
    else if (bNoData)
    {
//      FIFOWrite(0, 0, pep0state->pData);
//      by = M_CSR0_INPKTRDY | M_CSR0_DATAEND;
//      MWRITE_BYTE(M_REG_CSR0, by);
        /***清零 OUTPKTRDY 位,在发送时表示，将要发送的时最后一包数据；接收时将要接收的是最后一包数据；***/
        by = M_CSR0_SVDOUTPKTRDY | M_CSR0_DATAEND;
//                  pep0state->nBytesLeft = 0;
//                  pep0state->nState = M_EP0_TX;
    }
    else
    {
        /***清零 OUTPKTRDY 位***/
        by = M_CSR0_SVDOUTPKTRDY;
    }

    MWRITE_BYTE(M_REG_CSR0, by);

    return;

}


/****************************************
  USB Endpoint 0 Tx
  This routine is called when data is to be sent to the host.
****************************************/
static void USB_Endpoint0_Tx(PM_EP0_STATUS pep0state)
{
    int nBytes;
    BYTE    by;

    /* Determine number of bytes to send */
    if (pep0state->nBytesLeft <= M_EP0_MAXP)
    {
        nBytes = pep0state->nBytesLeft;
        pep0state->nBytesLeft = 0;
    }
    else
    {
        nBytes = M_EP0_MAXP;
        pep0state->nBytesLeft -= M_EP0_MAXP;
    }

    FIFOWrite(0, nBytes, pep0state->pData);
    pep0state->pData = (BYTE *)pep0state->pData + nBytes;
    if (nBytes < M_EP0_MAXP)
    {
        /***数据已经写入 FIFO 中,将要发送的时最后一包数据***/
        by = M_CSR0_INPKTRDY | M_CSR0_DATAEND;
        pep0state->nState = M_EP0_IDLE;
    }
    else
    {
        /***数据已经写入 FIFO 中***/
        by = M_CSR0_INPKTRDY;
    }

    MWRITE_BYTE(M_REG_CSR0, by);
}

/**
    *@USB_Endpoint0_Rx
    */
static void USB_Endpoint0_Rx(PM_EP0_STATUS pep0state)
{
    BYTE    byOutCSR;

    /* Set index register */
    MWRITE_BYTE(M_REG_INDEX, 0);
    /* Read OUT CSR register */
    byOutCSR = MREAD_BYTE(M_REG_CSR0);

    /* Get packet, */
    /* may need to unload more than one packet if double buffering enabled */
    /*端点FIFO 中有数据*/
    while (byOutCSR & M_CSR0_OUTPKTRDY)
    {
        pep0state->nState = M_EP0_IDLE;

        /* Unload FIFO */
        FIFORead(0, 1, pep0state->pData);//NUM CAP SCR指示灯信息
        /*Cear flag*/
        MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
        /* Check for another packet */
        byOutCSR = MREAD_BYTE(M_REG_CSR0);
    }

}
/****************************************
  ConfigureIfs
  This routine is called when a SET_CONFIGURATION or SET_INTERFACE
  command is received and will set the gpCurIF pointers
  to the selected interface descriptors and will set the
  maximum packet size and operating mode for the endpoints in
  the selected interfaces
****************************************/
static int ConfigureIfs(void)
{
    PSTD_CFG_DSCR   pcfg;
    BYTE        byIf, byAltIf, byEP, byNumEPs, by;
    BYTE        *pby;
    BYTE        *pbyIfVal;
    PSTD_IF_DSCR    pif;
    PSTD_EP_DSCR    pep;

    /* Set pointer to first interface descriptor in current configuration */
    pby = (BYTE *)(gpCurCfg) + sizeof(STD_CFG_DSCR);
    pif = (PSTD_IF_DSCR)pby;

    /* Loop through all interfaces in the current configuration */
    pcfg = (PSTD_CFG_DSCR)gpCurCfg;  //configure descriptor pointer
    pbyIfVal = (BYTE *)&gbyCurIfVal;    
    for (byIf = 0; byIf < pcfg->bNumInterfaces; byIf++, pbyIfVal++)
    {
        /* Advance pointer to selected alternate interface descriptor */
        if (*pbyIfVal)
        {
            for (byAltIf = 0; byAltIf < *pbyIfVal; byAltIf++)
            {
                byNumEPs = pif->bNumEndpoints;
                pby += sizeof(STD_IF_DSCR) + byNumEPs * sizeof(STD_EP_DSCR);
                pif = (PSTD_IF_DSCR)pby;
                /* Check an alternate setting > number of alternates not specified */
                if (!pif->bAlternateSetting)
                {
                    return FALSE;
                }
            }
        }

        /* Store pointer to interface in global array */
        gpCurIf[byIf] = pif;

        /* Loop through all endpoints in interface */
        byNumEPs = pif->bNumEndpoints;

        if (byIf == 0)
        {
            pby += sizeof(STD_IF_DSCR);
        }
        else if (byIf == 1)
        {
            pby += sizeof(STD_IF_DSCR) + 19 + 8 ;
        }
        else
        {
            pby += sizeof(STD_IF_DSCR);
        }

        for (byEP = 0; byEP < byNumEPs; byEP++)
        {
            pep = (PSTD_EP_DSCR)pby;

            /* Configure the endpoint */
            MWRITE_BYTE(M_REG_INDEX, (pep->bEndpointAddress & 0x0F));
            /* Round up max packet size to a multiple of 8 for writing to MaxP registers */
            by = (BYTE)((pep->wMaxPacketSize + 7) >> 3);
            if (pep->bEndpointAddress & 0x80)
            {
                MWRITE_BYTE(M_REG_INMAXP, by);
                by = MREAD_BYTE(M_REG_INCSR2);
                switch (pep->bmAttributes & M_EP_TFMASK)
                {
                case M_EP_ISO:
                    by |= M_INCSR2_ISO;
                    break;
                case M_EP_BULK:
                case M_EP_INTR:
                    by &= ~M_INCSR2_ISO;
                    break;
                }
                /* Set mode bit high (only strictly necessary if sharing a FIFO) */
                by |= M_INCSR2_MODE;
                MWRITE_BYTE(M_REG_INCSR2, by);
                /* Other configuration for an IN endpoint */
                /* e.g. AutoClr, DMA */
                /* should be added here */
            }
            else
            {
                MWRITE_BYTE(M_REG_OUTMAXP, by);
                by = MREAD_BYTE(M_REG_OUTCSR2);
                switch (pep->bmAttributes & M_EP_TFMASK)
                {
                case M_EP_ISO:
                    by |= M_OUTCSR2_ISO;
                    break;
                case M_EP_BULK:
                case M_EP_INTR:
                    by &= ~M_OUTCSR2_ISO;
                    break;
                }
                MWRITE_BYTE(M_REG_OUTCSR2, by);
                /* Other configuration for an OUT endpoint */
                /* e.g. AutoSet, DMA */
                /* should be added here */
            }

            pby += sizeof(STD_EP_DSCR);
        }

        if (*pby != USB_FSConfigDescriptor[USB_FSConfigDescriptorSize - 1])
        {
            /* Skip any alternate setting interfaces */
            if (byIf == 0)
            {
                pif = (PSTD_IF_DSCR)(pby + 8);
            }
            else if (byIf == 1)
            {
                pif = (PSTD_IF_DSCR)pby;
            }
        }
    }

    return TRUE;
}


void USB_Remote_Wakeup(void)
{
    USB->POWER |= 0x04;
    SysTick_Delay_Ms(12);
    USB->POWER &= ~0X04;
    SysTick_Delay_Ms(12);
}

/*********************** (C) COPYRIGHT FMD ********************END OF FILE****/
