/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_ep.c
* @author    : FMD AE
* @brief     : usb_ep program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "usb_ep.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ---------------------------------------------------------------*/
/******************************************************************************
  * @brief  EndpointBulkIn program
  * @param  pbistate
  *         nCallState
  * @note   EndpointBulkIn service routine
  *         This routine provides simple handling for bulk in endpoints.
  *         Endpoint MaxP is defined in endpoint.h.
  *         While the nBytesLeft field of the pbostate structure is >= 0
  *         the endpoint will send packets of MaxP, with the residue in the last packet.
  *         If the data to send is an exact multiple of MaxP then a null data packet
  *         will be sent.
  * @retval None
  *****************************************************************************
*/
void EndpointBulkIn(M_EPBIN_STATUS pbistate, int nCallState)
{
    int     nBytes;
    BYTE    byInCSR;

    /* Reset endpoint */
    if (nCallState == M_EP_RESET)
    {
        /*л˵*/
        MWRITE_BYTE(M_REG_INDEX, pbistate.byEP);
        /*Ӧ˵ݷת״̬FIFOҪ͵*/
        byInCSR = M_INCSR_CDT | M_INCSR_FF;
        MWRITE_BYTE(M_REG_INCSR1, byInCSR);
    }
    else
    {
        /* Check whether there is any data to send */
        if (pbistate.nBytesLeft != M_EP_NODATA)
        {
            /* Determine number of bytes to send */
            if (pbistate.nBytesLeft < M_EP_MAXP)
            {
                nBytes = pbistate.nBytesLeft;
                pbistate.nBytesLeft = M_EP_NODATA;
            }
            else
            {
                nBytes = M_EP_MAXP;
                pbistate.nBytesLeft -= M_EP_MAXP;
            }

            /* Load FIFO */
            FIFOWrite((int)pbistate.byEP, nBytes, pbistate.pData);
            pbistate.pData = (BYTE *)pbistate.pData + nBytes;

            MWRITE_BYTE(M_REG_INDEX, pbistate.byEP);
            /*ʾҪ͵ѾװӦ˵FIFO*/
            byInCSR = M_INCSR_IPR;
            MWRITE_BYTE(M_REG_INCSR1, byInCSR);
        }
    }
}


/******************************************************************************
  * @brief  EndpointBulkOut program
  * @param  pbistate
  *         nCallState
  * @note   EndpointBulkOut service routine
  *         This routine provides simple handling for bulk out endpoints.
  *         Endpoint MaxP is defined in endpoint.h.
  *         It assumes a transfer is complete when a packet of less than MaxP is received.
  * @retval None
  *****************************************************************************
*/
void EndpointBulkOut(M_EPBOUT_STATUS pbostate, int nCallState)
{
    int     nBytes;
    BYTE    byOutCSR;

    /* Reset endpoint */
    if (nCallState == M_EP_RESET)
    {
        /***ö˵***/
        MWRITE_BYTE(M_REG_INDEX, pbostate.byEP);
        byOutCSR = M_OUTCSR_CDT | M_OUTCSR_FF;
        /***λ˵ݷת״̬,ս FIFO е***/
        MWRITE_BYTE(M_REG_OUTCSR1, byOutCSR);
    }
    else
    {
        /* Set index register */
        MWRITE_BYTE(M_REG_INDEX, pbostate.byEP);

        /* Read OUT CSR register */
        byOutCSR = MREAD_BYTE(M_REG_OUTCSR1);

        /* Get packet, */
        /* may need to unload more than one packet if double buffering enabled */
        /*˵FIFO */
        while (byOutCSR & M_OUTCSR_OPR)
        {
            /* Get packet size */
            nBytes = (int)MREAD_BYTE(M_REG_OUTCOUNT2);
            nBytes <<= 8;
            nBytes |= (int)MREAD_BYTE(M_REG_OUTCOUNT1);

            /* Check there is room in the buffer */
            if (pbostate.nBytesRecv + nBytes > pbostate.nBuffLen)
            {
                /* Call to function to handle buffer over run */
                MWRITE_BYTE(M_REG_OUTCSR1, 0);
                return;
            }

            /* Unload FIFO */
            FIFORead((int)pbostate.byEP, nBytes, pbostate.pData);

            /* Update status */
            pbostate.nBytesRecv += nBytes;
            pbostate.pData = (BYTE *)pbostate.pData + nBytes;

            /* Check for end of transfer */
            if (nBytes <= M_EP_MAXP)
            {
                /* Call function to handle received data */
            }

            /* Clear OutPktRdy */
            MWRITE_BYTE(M_REG_OUTCSR1, 0);

            /* Check for another packet */
            byOutCSR = MREAD_BYTE(M_REG_OUTCSR1);
        }
    }
}
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/

